<?php

/**
 * @package 	 		: Relational
 * @author	 			: Pierre FROMAGER <pf at pier dash infor dot fr>
 * @contributors	 	: Pierre FROMAGER <pf at pier dash infor dot fr>
 * @release				: $Date: 2011/07/02 00:00:00 $
 * @purpose				: Getting relations between members.
 * @usage				: Ospf algorithm.
 * @algo				: Ospf algorithm.
 * @env					: Php 5.2.1.
 * @version $Revision	: 1.0 $
 * @license GNU GPL
 */

// Floyd Warshall Lib
require_once('floyd-warshall.class.php');

// Interface Declaration
interface relationInterface{
	function get($name);
	function getId($name);
	function set($rid,$name);
	function init();
	function load();
	function build();
	//function getInfo();
	function save();
}

// Abstract Class for Relation implements relationInterface
abstract class relation extends FloydWarshall implements relationInterface {
	public		$_source	= null;
	public 		$_relations	= Array();
	public  	$_members	= Array();
	public  	$_matrix	= Array();
	private 	$_tmp		= null;
	private		$_fw		= null;
	
		/*--------------------------------------------------
		 * Getter
		 * Get relation from name member
		 * in $name string
		 * out $idR array('rid'=>RelationId,'idName'=>NodeId)
		 --------------------------------------------------*/
		public function get($name){
			if (in_array($name,$this->_members)){$rt=$this->_relations[$name];} // $name knows this people
			else $rt=false;
			if (!$rt) {foreach($this->_relations as $key=>$value){
				if (in_array($name,$value)){$rt[]=$key;} // $name is knowed by
			}
			}
			return $rt;
		}

		function getId($name){
			$rt=false;
			if (in_array($name,$this->_members)){
				$rt=array_search($name,$this->_members);// return member id from $name
			}
			return $rt;
		}

		function getNodes() {
			return $this->_fw->nodes;
		}

		function getNodesname() {
			return  $this->_fw->nodenames;
		}
		
		function graphviz() {
			$content='';$cpt=-1;
			$header='digraph G {'."\n".
					'edge [color=red, arrowsize=1];'."\n".
					'node [color=lightyellow2, style=filled]'.";\n";	
			$s=sizeOf($this->_members);
			for($y=0;$y<=$s;$y++){
				for($x=0;$x<=$s;$x++){
					$connected=(($this->_matrix[$x][$y]==1)&&($x!=$y));
					if ($connected){$content.=$x.'->'.$y.";\n";}
				}
			}
			foreach($this->_members as $destmember){
				$cpt++;
				$footer.=$cpt.' [label="'.$destmember.'"];'."\n";
			}
			return $header.$content.$footer."\n".'}';
		}
		
		public function set($rid,$name){		
			$this->_relations[$rid][]=$name;
		$this->build();
		}
		
		public function update($rid,$oldname,$newname){		
			$knowns= $this->_relations[$rid];
			foreach ($knowns as $known){
				$c++;
				if ($this->_relations[$rid][$c]==$oldname) $this->_relations[$rid][$c]=$newname;				
			}			
		$this->build();
		}
		
		function load(){
			$container=file_get_contents($this->_source,FILE_USE_INCLUDE_PATH);
			$container=str_replace(CHR(13),'',$container);
			$container=str_replace(CHR(10),'',$container);
			// Init Tmp Array
			$this->_tmp=explode(';',$container);
			unset($container);
		}
		
		function init(){
			// Make _relations Array from _tmp
			for($c=0;$c<sizeOf($this->_tmp);$c++){
				$exp=explode(':',$this->_tmp[$c]);
				$this->_relations[$exp[0]]=explode(',',$exp[1]);
			}
				
			unset($this->_tmp); // clean _tmp
			// Make _members Array from _relations				
			$parents=array_keys($this->_relations);
			foreach($this->_relations as $parent=>$children){
				if (!in_array($parent,$this->_members)){$this->_members[]=$parent;}
				foreach($children as  $key=>$value){
				 if (!in_array($value,$this->_members)){$this->_members[]=$value;}
				}
			}
			array_pop($this->_members);
		}
		
		function build() {
			unset($this->_tmp);
			$aS=sizeOf($this->_members);
			// Make Square _members Relation Matrix Init to 0
			for($c=0;$c<$aS;$c++){
				for($cc=0;$cc<$aS;$cc++){
					$this->_tmp[$c][$cc]=0;}
			}
			// Build Relations
			for($c=0;$c<$aS;$c++){
				for($cc=0;$cc<$aS;$cc++){
					if ($c==$cc) {$this->_tmp[$c][$cc]=1;} // Each member knows himself
					else {
						if (in_array($this->_members[$c], $this->get($this->_members[$cc]))) {
							$this->_tmp[$c][$cc]=1;$this->_tmp[$cc][$c]=1;} // Mirrors Relation In Matrix
					}
				}
			}
			$this->_matrix=$this->_tmp;
			unset($this->_fw);
			$this->_fw=new FloydWarshall($this->_matrix,$this->_members);
			unset($this->_tmp);
		}

		public function getDistance($name1,$name2) {
			return $this->_fw->get_distance($this->getId($name1),$this->getId($name2));
		}
		
		public function getPath($name1,$name2) {
			$arPath=Array();
			$node_Name=Array();
			$arPath=$this->_fw->get_path($this->getId($name1),$this->getId($name2));
			foreach($arPath as $pathId){
				$node_Name[]=$this->_members[$pathId];
			}
			return implode(',',$node_Name);
		}
		
		public function showGraph(){
			$rt= '<table id="table_graph" class="managetable" border="1" style="font-size:9px">';
			$rt.= '<tr style="text-align:center;"><td> </td><td>'.implode('</td><td>',$this->_members).'</td></tr>';
			for($y=0;$y<sizeof($this->_members);$y++){
				$rt.= '<tr style="text-align:center">';
				$rt.= '<td>'.$this->_members[$y].'</td>';
				for($x=0;$x<sizeof($this->_members);$x++){$rt.= '<td>'.$this->_matrix[$x][$y].'</td>';}
			}
			$rt.= '</tr>';
			$rt.= '</table><br/>';
			return $rt;
		}

		public function showDist(){
			$rt='';
			$rt.=$this->_fw->print_dist();
			return $rt;
		}
		
		public function showPred(){
			$rt='';
			$rt.=$this->_fw->print_pred();
			return $rt;
		}
		
			
		function save(){
			$fileContent='';
			$edges=array_keys($this->_relations);
			foreach($edges as $edge){
				$s=count($this->_relations[$edge]);
				$line='';$line.=$edge.':';
				for($c=0;$c<$s;$c++){$line.=$this->_relations[$edge][$c].',';}
				$line=substr($line,0,-1);$fileContent.=$line.";\n";$line='';
			}
			$isWrited=file_put_contents($this->_source, $fileContent,FILE_USE_INCLUDE_PATH);
			return $isWrited;
		}
		
	
}
?>
